@@ -10,7 +10,20 @@ from .base import GenericAPI, ERROR_THRESHOLD
 from .. import utils
 
 
-class UploadProgressHandler:
+class FileTransferProgressHandler:
+    """
+    Invokes a callback when transfer size thresholds have been reached.
+
+    Parameters
+    ----------
+    chunk_size : int
+        The constant interval between size thresholds.
+    total_size : int
+        The total size of a file in bytes.
+    callback : callable
+        The callback to be invoked when a size threshold has been crossed.
+    """
+
     def __init__(self, chunk_size, total_size, callback):
         if not isinstance(chunk_size, int) or chunk_size <= 0:
             raise ValueError("chunk_size must be a positive integer")
@@ -32,8 +45,17 @@ class UploadProgressHandler:
         self.callback_threshold = chunk_size
         
     def add(self, increment):
-        # increment must be a positive integer, though in the interest of performance
-        # that won't be checked.
+        """
+        Update total size (number of bytes transferred) by increment.
+
+        self.callback will be invoked if the addition of the new increment
+        crosses a size threshold.
+
+        Parameters
+        ----------
+        increment : int
+            The number of bytes transferred since the last invocation.
+        """
         if not increment:
             return
 
@@ -601,7 +623,7 @@ class FilesAPI(GenericAPI):
         # Return the file id.
         return (file_id, file_name,)
 
-    def download(self, file_id, directory):
+    def download(self, file_id, directory, progress_callback=None, n_chunks=100):
         """
         Download a file from HyperThought to the local file system.
 
@@ -611,6 +633,12 @@ class FilesAPI(GenericAPI):
             The HyperThought id for a file to be downloaded.
         directory : str
             A local directory path to which the file will be downloaded.
+        progress_callback : callable (int -> None) or None
+            A callback for handling upload progress.  Will be called each time
+            a given number of bytes (chunk) is uploaded.
+        n_chunks : int
+            The number of chunks to be handled by progress_callback.
+            Will be ignored if progress_callback is None.
         """
         # Validate parameters.
         self._validate_id(file_id)
@@ -626,6 +654,7 @@ class FilesAPI(GenericAPI):
 
         # Get the file name.
         file_document = self.get_document(id=file_id)
+        file_size = file_document['content']['size']
         file_name = file_document['content']['name']
         file_path = os.path.join(active_directory_path, file_name)
 
@@ -633,7 +662,13 @@ class FilesAPI(GenericAPI):
         url = self._get_download_url(file_id)
 
         # Use the url to download the file.
-        self._download_using_url(url, file_path)
+        self._download_using_url(
+            download_url=url,
+            local_path=file_path,
+            file_size=file_size,
+            progress_callback=progress_callback,
+            n_chunks=n_chunks,
+        )
 
     def delete(self, id):
         """
@@ -830,7 +865,7 @@ class FilesAPI(GenericAPI):
 
         if progress_callback is not None:
             chunk_size = math.ceil(file_size / n_chunks)
-            progress_handler = UploadProgressHandler(
+            progress_handler = FileTransferProgressHandler(
                 chunk_size=chunk_size,
                 total_size=file_size,
                 callback=progress_callback,
@@ -926,7 +961,8 @@ class FilesAPI(GenericAPI):
 
         return r.json()['url']
 
-    def _download_using_url(self, download_url, local_path):
+    def _download_using_url(self, download_url, local_path, file_size,
+                            progress_callback, n_chunks):
         """
         Use a generated url to download a file.
 
@@ -937,6 +973,12 @@ class FilesAPI(GenericAPI):
             See self._get_download_url. 
         local_path : str
             The local system path where the downloaded file will be saved.
+        progress_callback : callable (int -> None) or None
+            A callback for handling upload progress.  Will be called each time
+            a given number of bytes (chunk) is uploaded.
+        n_chunks : int
+            The number of chunks to be handled by progress_callback.
+            Will be ignored if progress_callback is None.
         """
         kwargs = {
             'url': download_url,
@@ -948,13 +990,29 @@ class FilesAPI(GenericAPI):
             kwargs['headers'] = self._auth.get_headers()
             kwargs['cookies'] = self._auth.get_cookies()
 
+        progress_handler = None
+
+        if progress_callback is not None:
+            progress_chunk_size = math.ceil(file_size / n_chunks)
+            progress_handler = FileTransferProgressHandler(
+                chunk_size=progress_chunk_size,
+                total_size=file_size,
+                callback=progress_callback,
+            )
+
+        DOWNLOAD_CHUNK_SIZE = 8192
+
         with requests.get(**kwargs) as r:
             r.raise_for_status()
+
             with open(local_path, 'wb') as f:
-                for chunk in r.iter_content(chunk_size=8192):
+                for chunk in r.iter_content(chunk_size=DOWNLOAD_CHUNK_SIZE):
                     if chunk:   # filter out keep-alive new chunks
                         f.write(chunk)
 
+                        if progress_handler:
+                            progress_handler.add(DOWNLOAD_CHUNK_SIZE)
+
     def _validate_id(self, id_):
         assert isinstance(id_, str), f"expected string as id, found {type(id_)}"
         return id_
