@@ -1,14 +1,6 @@
-import os
-import sys
-
 import requests
 
-up = os.path.dirname
-# <submodule>/api/workflow/common.py
-submodule_directory = up(up(up(__file__)))
-sys.path.append(submodule_directory)
-
-from api.base import GenericAPI, ERROR_THRESHOLD
+from .base import GenericAPI, ERROR_THRESHOLD
 
 
 class MetatronAPI(GenericAPI):
